@@ -26,6 +26,7 @@
 # endif
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -78,6 +79,7 @@
 #include <QTableWidget>
 #include <QTableWidgetItem>
 #include <QTabWidget>
+#include <QTextDocument>
 #include <QTimer>
 #include <QVBoxLayout>
 #include <QVector>
@@ -141,6 +143,11 @@ QDjViewErrorDialog::error(QString msg, QString, int)
   // Remove [1-nnnnn] prefix from djvulibre-3.5
   if (msg.startsWith("["))
     msg = msg.replace(QRegExp("^\\[\\d*-?\\d*\\]\\s*") , "");
+#if QT_VERSION >= 0x50000
+  msg = msg.toHtmlEscaped();
+#else
+  msg = Qt::escape(msg);
+#endif
   // Ignore empty and duplicate messages
   if (msg.isEmpty()) return;
   if (!d->messages.isEmpty() && msg == d->messages[0]) return;
@@ -170,6 +177,66 @@ QDjViewErrorDialog::done(int result)
 }
 
 
+// =======================================
+// QDJVIEWAUTHDIALOG
+// =======================================
+
+#include "ui_qdjviewauthdialog.h"
+
+struct QDjViewAuthDialog::Private
+{
+  Ui_QDjViewAuthDialog ui;
+};
+
+QDjViewAuthDialog::~QDjViewAuthDialog()
+{
+  delete d;
+}
+
+QDjViewAuthDialog::QDjViewAuthDialog(QWidget *parent)
+  : QDialog(parent), 
+    d(new Private)
+{
+  d->ui.setupUi(this);
+  setWindowTitle(tr("Authentication required - DjView"));
+}
+
+QString 
+QDjViewAuthDialog::user() const
+{
+  return d->ui.userLineEdit->text();
+}
+
+QString 
+QDjViewAuthDialog::pass() const
+{
+  return d->ui.passLineEdit->text();
+}
+
+void 
+QDjViewAuthDialog::setInfo(QString why)
+{
+#if QT_VERSION >= 0x50000
+  QString ewhy = why.toHtmlEscaped();
+#else
+  QString ewhy = Qt::escape(why);
+#endif
+  QString txt = QString("<html>%1</html>").arg(ewhy);
+  d->ui.whyLabel->setText(txt);
+}
+
+void 
+QDjViewAuthDialog::setUser(QString user)
+{
+  d->ui.userLineEdit->setText(user);
+}
+
+void 
+QDjViewAuthDialog::setPass(QString pass)
+{
+  d->ui.passLineEdit->setText(pass);
+}
+
 
 
 // =======================================
@@ -432,7 +499,7 @@ QDjViewInfoDialog::fillFileCombo()
       if (info.type == 'P')
         {
           if (info.title && info.name && strcmp(info.title, info.name))
-            msg = tr("Page #%1 - \253 %2 \273") // << .. >>
+            msg = trUtf8("Page #%1 - \302\253 %2 \302\273") // << .. >>
               .arg(info.pageno + 1).arg(QString::fromUtf8(info.title));
           else
             msg = tr("Page #%1").arg(info.pageno + 1);
@@ -597,13 +664,21 @@ QDjViewMetaDialog::QDjViewMetaDialog(QDjView *parent)
   d->ui.docTable->setHorizontalHeaderLabels(labels);
   d->ui.docTable->horizontalHeader()->setHighlightSections(false);
   d->ui.docTable->horizontalHeader()->setStretchLastSection(true);
+#if QT_VERSION >= 0x50000
+  d->ui.docTable->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
+#else
   d->ui.docTable->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
+#endif
   d->ui.docTable->verticalHeader()->hide();
   d->ui.pageTable->setColumnCount(2);
   d->ui.pageTable->setHorizontalHeaderLabels(labels);
   d->ui.pageTable->horizontalHeader()->setHighlightSections(false);
   d->ui.pageTable->horizontalHeader()->setStretchLastSection(true);
+#if QT_VERSION >= 0x50000
+  d->ui.pageTable->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
+#else
   d->ui.pageTable->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
+#endif
   d->ui.pageTable->verticalHeader()->hide();
   d->ui.pageCombo->setEnabled(false);
   d->ui.jumpButton->setEnabled(false);
@@ -1474,7 +1549,6 @@ struct QDjViewPrintDialog::Private
   QDjViewExporter *exporter;
   QString groupname;
   QPrinter *printer;
-  QPrintDialog *dialog;
   bool stopping;
 };
 
@@ -1498,7 +1572,6 @@ QDjViewPrintDialog::QDjViewPrintDialog(QDjView *djview)
   d->stopping = false;
   d->exporter = 0;
   d->printer = new QPrinter(QPrinter::HighResolution);
-  d->dialog = new QPrintDialog(d->printer, this);
   d->ui.setupUi(this);
   setAttribute(Qt::WA_GroupLeader, true);
   connect(d->ui.okButton, SIGNAL(clicked()), 
@@ -1558,13 +1631,9 @@ QDjViewPrintDialog::QDjViewPrintDialog(QDjView *djview)
     d->printer->setPageOrder(QPrinter::FirstPageFirst);
   
   // Create exporter
-#ifdef Q_WS_WIN
-  // ... maybe do windows specific things here.
+#ifdef Q_OS_WIN
   if (! d->exporter)
     d->exporter = QDjViewExporter::create(this, d->djview, "PRN");
-#endif
-#ifdef Q_WS_MAC
-  // ... maybe do mac specific things here.
 #endif
   if (! d->exporter)
     d->exporter = QDjViewExporter::create(this, d->djview, "PS");
@@ -1672,18 +1741,52 @@ QDjViewPrintDialog::choose()
 {
   if (d->exporter)
     {
-      d->exporter->printSetup(d->dialog, false);
-#if QT_VERSION >= 0x40400
-      delete d->dialog;
-      d->dialog = new QPrintDialog(d->printer, this);
-#endif
-      QPrintDialog::PrintDialogOptions options = d->dialog->enabledOptions();
+      d->exporter->savePrintSetup(d->printer);
+      QPrintDialog *dialog = new QPrintDialog(d->printer, this);
+      // options
+      QPrintDialog::PrintDialogOptions options = dialog->enabledOptions();
       options |= QPrintDialog::PrintCollateCopies;
       options &= ~QPrintDialog::PrintToFile;
       options &= ~QPrintDialog::PrintSelection;
-      d->dialog->setEnabledOptions(options);
-      if (d->dialog->exec() == QDialog::Accepted)
-        d->exporter->printSetup(d->dialog, true);      
+      dialog->setEnabledOptions(options);
+      // copy page spec
+      int pagenum = d->djview->pageNum();
+      int curpage = d->djview->getDjVuWidget()->page();
+      int frompage = d->ui.fromPageCombo->currentIndex();
+      int topage = d->ui.toPageCombo->currentIndex();
+      dialog->setMinMax(1, pagenum);
+      dialog->setFromTo(1, pagenum);
+      dialog->setPrintRange(QPrintDialog::PageRange);
+      if (d->ui.currentPageButton->isChecked())
+        dialog->setFromTo(curpage+1, curpage+1);
+      else if (d->ui.pageRangeButton->isChecked())
+        dialog->setFromTo(frompage+1, topage+1);
+      else
+        dialog->setPrintRange(QPrintDialog::AllPages);
+      // exec
+      if (dialog->exec() == QDialog::Accepted)
+        {
+          d->exporter->loadPrintSetup(d->printer, dialog);
+          // copy page spec back
+          frompage = qBound(0, dialog->fromPage()-1, pagenum-1);
+          topage = qBound(0, dialog->toPage()-1, pagenum-1);
+          if (dialog->printRange() == QPrintDialog::AllPages)
+            {
+              d->ui.documentButton->setChecked(true);
+              d->ui.fromPageCombo->setCurrentIndex(0);
+              d->ui.toPageCombo->setCurrentIndex(pagenum-1);
+            }
+          else
+            {
+              d->ui.fromPageCombo->setCurrentIndex(frompage);
+              d->ui.toPageCombo->setCurrentIndex(topage);
+              if (frompage == curpage && topage == curpage)
+                d->ui.currentPageButton->setChecked(true);
+              else
+                d->ui.pageRangeButton->setChecked(true);
+            }
+        }
+      delete dialog;
     }
   refresh();
 }
