@@ -28,6 +28,7 @@
 # endif
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -90,7 +91,11 @@
 #include "qdjvuwidget.h"
 #include "qdjvu.h"
 
-
+#ifdef WIN32
+# define wdup(fh) _dup(fh)
+#else
+# define wdup(fh) (fh)
+#endif
 
 // ----------------------------------------
 // FACTORY
@@ -171,18 +176,37 @@ QDjViewExporter::exportOnePageOnly()
 void 
 QDjViewExporter::resetProperties() 
 { 
+  // reset all properties to default values
 }
 
 
 void 
 QDjViewExporter::loadProperties(QString) 
 { 
+  // fill properties from settings keyed by string
 }
 
 
 void 
 QDjViewExporter::saveProperties(QString) 
 { 
+  // save properties into settings keyed by string
+}
+
+
+bool
+QDjViewExporter::loadPrintSetup(QPrinter*, QPrintDialog*) 
+{ 
+  // fill properties using printer and print dialog data
+  return false;
+}
+
+
+bool
+QDjViewExporter::savePrintSetup(QPrinter*) 
+{ 
+  // save properties into printer settings
+  return false;
 }
 
 
@@ -229,13 +253,6 @@ QDjViewExporter::name()
 }
 
 
-bool
-QDjViewExporter::printSetup(QPrintDialog *, bool) 
-{ 
-  return false;
-}
-
-
 bool
 QDjViewExporter::print(QPrinter *) 
 { 
@@ -345,25 +362,21 @@ QDjViewDjVuExporter::QDjViewDjVuExporter(QDialog *parent, QDjView *djview,
 
 QDjViewDjVuExporter::~QDjViewDjVuExporter()
 {
-  switch(status())
-    {
-    case DDJVU_JOB_STARTED:
-      if (job)
-        ddjvu_job_stop(*job);
-    case DDJVU_JOB_FAILED:
-    case DDJVU_JOB_STOPPED:
-      if (file.openMode() & (QIODevice::WriteOnly|QIODevice::Append))
-        file.remove();
-    default:
-      break;
-    }
+  QIODevice::OpenMode mode = file.openMode();
+  ddjvu_status_t st = status();
+  if (st == DDJVU_JOB_STARTED)
+	if (job)
+      ddjvu_job_stop(*job);
   if (output)
     ::fclose(output);
+  output = 0;
   if (file.openMode())
     file.close();
+  if (st != DDJVU_JOB_OK)
+      if (mode & (QIODevice::WriteOnly|QIODevice::Append))
+        file.remove();
   if (djview)
     ddjvu_cache_clear(djview->getDjVuContext());
-  output = 0;
 }
 
 
@@ -409,7 +422,7 @@ QDjViewDjVuExporter::save(QString fname)
   file.setFileName(fname);
   file.remove();
   if (file.open(QIODevice::WriteOnly))
-    output = ::fdopen(file.handle(), "wb");
+    output = ::fdopen(wdup(file.handle()), "wb");
   if (! output)
     {
       failed = true;
@@ -490,10 +503,11 @@ public:
   virtual void resetProperties();
   virtual void loadProperties(QString group);
   virtual void saveProperties(QString group);
+  virtual bool loadPrintSetup(QPrinter *printer, QPrintDialog *dialog);
+  virtual bool savePrintSetup(QPrinter *printer);
   virtual int propertyPages();
   virtual QWidget* propertyPage(int num);
   virtual bool save(QString fileName);
-  virtual bool printSetup(QPrintDialog *dialog, bool dir);
   virtual bool print(QPrinter *printer);
   virtual ddjvu_status_t status();
   virtual void stop();
@@ -552,19 +566,15 @@ QDjViewPSExporter::setup()
 QDjViewPSExporter::~QDjViewPSExporter()
 {
   // cleanup output
-  switch(status())
-    {
-    case DDJVU_JOB_STARTED:
+  QIODevice::OpenMode mode = file.openMode();
+  ddjvu_status_t st = status();
+  if (st == DDJVU_JOB_STARTED)
       if (job)
         ddjvu_job_stop(*job);
-    case DDJVU_JOB_FAILED:
-    case DDJVU_JOB_STOPPED:
-      if (file.openMode() & (QIODevice::WriteOnly|QIODevice::Append))
-        file.remove();
-    default:
-      break;
-    }
   closeFile();
+  if (st != DDJVU_JOB_OK)
+	if (mode & (QIODevice::WriteOnly|QIODevice::Append))
+        file.remove();
   // delete property pages
   delete page1;
   delete page2;
@@ -614,7 +624,7 @@ QDjViewPSExporter::QDjViewPSExporter(QDialog *parent, QDjView *djview,
                          "printers. Level 3 print color document faster "
                          "on recent printers.</html>") );
   page2->setWhatsThis(tr("<html><b>Position and scaling.</b><br>"
-                         "Option <tt>Scale to fit</tt> accomodates "
+                         "Option <tt>Scale to fit</tt> accommodates "
                          "whatever paper size your printer uses. "
                          "Zoom factor <tt>100%</tt> reproduces the initial "
                          "document size. Orientation <tt>Automatic</tt> "
@@ -729,6 +739,55 @@ QDjViewPSExporter::saveProperties(QString group)
 }
 
 
+bool
+QDjViewPSExporter::loadPrintSetup(QPrinter *printer, QPrintDialog *dialog)
+{
+  bool grayscale = (printer->colorMode() == QPrinter::GrayScale);
+  bool landscape = (printer->orientation() == QPrinter::Landscape);
+  ui1.grayScaleButton->setChecked(grayscale);  
+  ui1.colorButton->setChecked(!grayscale);  
+  ui2.landscapeButton->setChecked(landscape);
+  ui2.portraitButton->setChecked(!landscape);
+  // BEGIN HACK //
+  // Directly steal settings from print dialog.
+  copies = 1;
+  collate = true;
+  lastfirst = false;
+#if QT_VERSION >= 0x50000
+  QSpinBox* lcop = dialog->findChild<QSpinBox*>("copies");
+  QCheckBox* lcol = dialog->findChild<QCheckBox*>("collate");
+  QCheckBox* lplf = dialog->findChild<QCheckBox*>("reverse");
+#elif QT_VERSION >= 0x40400
+  QSpinBox* lcop = qFindChild<QSpinBox*>(dialog, "copies");
+  QCheckBox* lcol = qFindChild<QCheckBox*>(dialog, "collate");
+  QCheckBox* lplf = qFindChild<QCheckBox*>(dialog, "reverse");
+#else
+  QSpinBox* lcop = qFindChild<QSpinBox*>(dialog, "sbNumCopies");
+  QCheckBox* lcol = qFindChild<QCheckBox*>(dialog, "chbCollate");
+  QCheckBox* lplf = qFindChild<QCheckBox*>(dialog, "chbPrintLastFirst");
+#endif
+  if (lcop)
+    copies = qMax(1, lcop->value());
+  if (lcol)
+    collate = lcol->isChecked();
+  if (lplf)
+    lastfirst = lplf->isChecked();
+  // END HACK //
+  return true;
+}
+
+
+bool
+QDjViewPSExporter::savePrintSetup(QPrinter *printer)
+{
+  bool g = ui1.grayScaleButton->isChecked();
+  bool l = ui2.landscapeButton->isChecked();
+  printer->setColorMode(g ? QPrinter::GrayScale : QPrinter::Color);
+  printer->setOrientation(l ? QPrinter::Landscape : QPrinter::Portrait);
+  return true;
+}
+
+
 int 
 QDjViewPSExporter::propertyPages()
 {
@@ -782,7 +841,7 @@ QDjViewPSExporter::openFile()
     {
       file.remove();
       if (file.open(QIODevice::WriteOnly))
-        output = ::fdopen(file.handle(), "wb");
+        output = ::fdopen(wdup(file.handle()), "wb");
     }
   else if (printer)
     {
@@ -971,52 +1030,6 @@ QDjViewPSExporter::print(QPrinter *qprinter)
   return start();
 }
 
-bool
-QDjViewPSExporter::printSetup(QPrintDialog *dialog, bool dir)
-{
-  printer = dialog->printer();
-  if (dir)
-    {
-      bool grayscale = (printer->colorMode() == QPrinter::GrayScale);
-      bool landscape = (printer->orientation() == QPrinter::Landscape);
-      ui1.grayScaleButton->setChecked(grayscale);  
-      ui1.colorButton->setChecked(!grayscale);  
-      ui2.landscapeButton->setChecked(landscape);
-      ui2.portraitButton->setChecked(!landscape);
-      // BEGIN HACK //
-      // Directly steal settings from print dialog.
-      copies = 1;
-      collate = true;
-      lastfirst = false;
-#if QT_VERSION >= 0x40400
-      QSpinBox* lcop = qFindChild<QSpinBox*>(dialog, "copies");
-      QCheckBox* lcol = qFindChild<QCheckBox*>(dialog, "collate");
-      QCheckBox* lplf = qFindChild<QCheckBox*>(dialog, "reverse");
-#else
-      QSpinBox* lcop = qFindChild<QSpinBox*>(dialog, "sbNumCopies");
-      QCheckBox* lcol = qFindChild<QCheckBox*>(dialog, "chbCollate");
-      QCheckBox* lplf = qFindChild<QCheckBox*>(dialog, "chbPrintLastFirst");
-#endif
-      if (lcop)
-        copies = qMax(1, lcop->value());
-      if (lcol)
-        collate = lcol->isChecked();
-      if (lplf)
-        lastfirst = lplf->isChecked();
-      // END HACK //
-    }
-  else
-    {
-      bool grayscale = ui1.grayScaleButton->isChecked();
-      bool landscape = ui2.landscapeButton->isChecked();
-      printer->setColorMode(grayscale ? 
-                            QPrinter::GrayScale : QPrinter::Color);
-      printer->setOrientation(landscape ? 
-                              QPrinter::Landscape : QPrinter::Portrait);
-    }
-  return true;
-}
-
 
 bool 
 QDjViewPSExporter::start()
@@ -1478,8 +1491,10 @@ QDjViewTiffExporter::closeFile()
     TIFFClose(tiff);
 #endif
   tiff = 0;
+  QIODevice::OpenMode mode = file.openMode();
+  file.close();
   if (curStatus > DDJVU_JOB_OK)
-    if (file.openMode() & (QIODevice::WriteOnly|QIODevice::Append))
+    if (mode & (QIODevice::WriteOnly|QIODevice::Append))
       file.remove();
 }
 
@@ -1509,7 +1524,7 @@ QDjViewTiffExporter::doPage()
   if (tiff)
     TIFFWriteDirectory(tiff);
   else if (file.open(QIODevice::ReadWrite))
-    tiff = TIFFFdOpen(file.handle(), file.fileName().toLocal8Bit().data(),"w");
+    tiff = TIFFFdOpen(wdup(file.handle()), file.fileName().toLocal8Bit().data(),"w");
   if (!tiff)
     message = tr("Cannot open output file.");
   else
@@ -1732,9 +1747,9 @@ QDjViewPdfExporter::doFinal()
   QByteArray onameArray = pdfFile.fileName().toLocal8Bit();
   file.close();
   if (file.open(QIODevice::ReadOnly))
-    input = TIFFFdOpen(file.handle(), inameArray.data(), "r");
+    input = TIFFFdOpen(wdup(file.handle()), inameArray.data(), "r");
   if (pdfFile.open(QIODevice::WriteOnly))
-    output = fdopen(pdfFile.handle(),"wb");
+    output = ::fdopen(wdup(pdfFile.handle()),"wb");
   if (input && output)
     {
       const char *argv[3];
@@ -1761,6 +1776,8 @@ QDjViewPdfExporter::doFinal()
     file.close();
   if (pdfFile.openMode())
     pdfFile.close();
+  if (tempFile.openMode())
+    tempFile.close();
   if (tempFile.exists())
     tempFile.remove();
 #else
@@ -1956,10 +1973,11 @@ public:
   virtual void resetProperties();
   virtual void loadProperties(QString group);
   virtual void saveProperties(QString group);
+  virtual bool loadPrintSetup(QPrinter *printer, QPrintDialog *dialog);
+  virtual bool savePrintSetup(QPrinter *printer);
   virtual int propertyPages();
   virtual QWidget* propertyPage(int num);
   virtual bool save(QString fileName);
-  virtual bool printSetup(QPrintDialog *dialog, bool dir);
   virtual bool print(QPrinter *printer);
 protected:
   virtual void closeFile();
@@ -2012,7 +2030,7 @@ QDjViewPrnExporter::QDjViewPrnExporter(QDialog *parent, QDjView *djview,
   page->setWhatsThis(tr("<html><b>Printing options.</b><br>"
                         "Option <tt>Color</tt> enables color printing. "
                         "Document pages can be decorated with a frame. "
-                        "Option <tt>Scale to fit</tt> accomodates "
+                        "Option <tt>Scale to fit</tt> accommodates "
                         "whatever paper size your printer uses. "
                         "Zoom factor <tt>100%</tt> reproduces the initial "
                         "document size. Orientation <tt>Automatic</tt> "
@@ -2073,25 +2091,25 @@ QDjViewPrnExporter::saveProperties(QString group)
 
 
 bool
-QDjViewPrnExporter::printSetup(QPrintDialog *dialog, bool dir)
+QDjViewPrnExporter::loadPrintSetup(QPrinter *printer, QPrintDialog *)
 {
-  printer = dialog->printer();
-  if (dir)
-    {
-      bool grayscale = (printer->colorMode() == QPrinter::GrayScale);
-      bool landscape = (printer->orientation() == QPrinter::Landscape);
-      ui.grayScaleButton->setChecked(grayscale);  
-      ui.colorButton->setChecked(!grayscale);  
-      ui.landscapeButton->setChecked(landscape);
-      ui.portraitButton->setChecked(!landscape);
-    }
-  else
-    {
-      bool grayscale = ui.grayScaleButton->isChecked();
-      bool landscape = ui.landscapeButton->isChecked();
-      printer->setColorMode(grayscale ? QPrinter::GrayScale : QPrinter::Color);
-      printer->setOrientation(landscape ? QPrinter::Landscape : QPrinter::Portrait);
-    }
+  bool grayscale = (printer->colorMode() == QPrinter::GrayScale);
+  bool landscape = (printer->orientation() == QPrinter::Landscape);
+  ui.grayScaleButton->setChecked(grayscale);  
+  ui.colorButton->setChecked(!grayscale);  
+  ui.landscapeButton->setChecked(landscape);
+  ui.portraitButton->setChecked(!landscape);
+  return true;
+}
+
+
+bool
+QDjViewPrnExporter::savePrintSetup(QPrinter *printer)
+{
+  bool grayscale = ui.grayScaleButton->isChecked();
+  bool landscape = ui.landscapeButton->isChecked();
+  printer->setColorMode(grayscale ? QPrinter::GrayScale : QPrinter::Color);
+  printer->setOrientation(landscape ? QPrinter::Landscape : QPrinter::Portrait);
   return true;
 }
 
@@ -2119,8 +2137,8 @@ QDjViewPrnExporter::save(QString fileName)
   this->fileName = fileName;
   printer = new QPrinter(QPrinter::HighResolution);
   printer->setOutputFileName(fileName);
-#if defined(Q_OS_UNIX) && !defined(Q_WS_MAC) 
-  printer->setOutputFormat(QPrinter::PostScriptFormat);
+#if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN) 
+  printer->setOutputFormat(QPrinter::PdfFormat);
 #endif
   return start();
 }
@@ -2211,7 +2229,11 @@ QDjViewPrnExporter::doPage()
     }
   else if (imageFormat == QImage::Format_Indexed8)
     {
+#if QT_VERSION >= 0x40600
+      img.setColorCount(256);
+#else
       img.setNumColors(256);
+#endif
       for (int i=0; i<256; i++)
         img.setColor(i, qRgb(i,i,i));
       fmt = ddjvu_format_create(DDJVU_FORMAT_GREY8, 0, 0);
