@@ -19,6 +19,7 @@
 # include "config.h"
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <math.h>
 #include <libdjvu/miniexp.h>
@@ -39,12 +40,14 @@
 #include <QRegExpValidator>
 #include <QSettings>
 #include <QStringList>
+#include <QTranslator>
 #include <QVariant>
 #include <QWhatsThis>
 
 #include "qdjviewprefs.h"
 #include "qdjview.h"
 #include "qdjvu.h"
+#include "djview.h"
 
 
 
@@ -123,7 +126,9 @@ QDjViewPrefs::instance(void)
 QDjViewPrefs::Saved::Saved(void)
   : remember(true),
     options(defaultOptions),
-    zoom(QDjVuWidget::ZOOM_FITWIDTH)
+    zoom(QDjVuWidget::ZOOM_FITWIDTH),
+    nsBorderBrush(QBrush(Qt::lightGray)),
+    nsBorderSize(8)
 {
 }
 
@@ -131,6 +136,7 @@ QDjViewPrefs::Saved::Saved(void)
 QDjViewPrefs::QDjViewPrefs(void)
   : QObject(QCoreApplication::instance()),
     windowSize(640,400),
+    windowMaximized(false),
     tools(defaultTools),
     gamma(2.2),
     white(QColor(Qt::white)),
@@ -139,8 +145,14 @@ QDjViewPrefs::QDjViewPrefs(void)
     pixelCacheSize(256*1024),
     lensSize(300),
     lensPower(3),
+    enableAnimations(true),
     advancedFeatures(false),
     showTextLabel(false),
+    invertLuminance(false),
+    mouseWheelZoom(false),
+    restrictOverride(false),
+    openGLAccel(false),
+    slideShowDelay(10),
     modifiersForLens(Qt::ControlModifier|Qt::ShiftModifier),
     modifiersForSelect(Qt::ControlModifier),
     modifiersForLinks(Qt::ShiftModifier),
@@ -157,9 +169,16 @@ QDjViewPrefs::QDjViewPrefs(void)
 {
   Options mss = (SHOW_MENUBAR|SHOW_STATUSBAR|SHOW_SIDEBAR);
   forFullScreen.options &= ~(mss|SHOW_SCROLLBARS|SHOW_TOOLBAR|SHOW_SIDEBAR);
+  forSlideShow = forFullScreen;
+  forSlideShow.zoom = QDjVuWidget::ZOOM_FITPAGE;
+  forSlideShow.remember = false;
+  forSlideShow.nsBorderBrush = QBrush(Qt::black);
   forFullPagePlugin.options &= ~(mss);
   forEmbeddedPlugin.options &= ~(mss|SHOW_TOOLBAR);
   forEmbeddedPlugin.remember = false;
+  forEmbeddedPlugin.zoom = 100;
+  forEmbeddedPlugin.nsBorderBrush = QBrush(Qt::white);
+  forEmbeddedPlugin.nsBorderSize = 0;
 }
 
 
@@ -183,11 +202,9 @@ QString
 QDjViewPrefs::modifiersToString(Qt::KeyboardModifiers m)
 {
   QStringList l;
-#ifdef Q_WS_MAC
+#ifdef Q_OS_DARWIN
   if (m & Qt::MetaModifier)
     l << "Control";
-  if (m & Qt::AltModifier)
-    l << "Alt";
   if (m & Qt::ControlModifier)
     l << "Command";
 #else
@@ -195,9 +212,9 @@ QDjViewPrefs::modifiersToString(Qt::KeyboardModifiers m)
     l << "Meta";
   if (m & Qt::ControlModifier)
     l << "Control";
+#endif
   if (m & Qt::AltModifier)
     l << "Alt";
-#endif
   if (m & Qt::ShiftModifier)
     l << "Shift";
   return l.join("+");
@@ -215,7 +232,9 @@ QDjViewPrefs::stringToModifiers(QString s)
       key = key.toLower();
       if (key == "shift")
         m |= Qt::ShiftModifier;
-#ifdef Q_WS_MAC
+      else if (key == "alt")
+        m |= Qt::AltModifier;
+#ifdef Q_OS_DARWIN
       else if (key == "control")
         m |= Qt::MetaModifier;
       else if (key == "command")
@@ -226,8 +245,6 @@ QDjViewPrefs::stringToModifiers(QString s)
       else if (key == "control")
         m |= Qt::ControlModifier;
 #endif
-      else if (key == "alt")
-        m |= Qt::AltModifier;
     }
   return m;
 }
@@ -299,24 +316,38 @@ QDjViewPrefs::loadGroup(QSettings &s, QString name, Saved &saved)
 }
 
 
+/*! Load recent files */
+
+void
+QDjViewPrefs::loadRecent()
+{
+  QSettings s;
+  if (s.contains("recentFiles"))
+    recentFiles = s.value("recentFiles").toStringList();
+}
+
 /*! Load saved preferences and set all member variables. */
 
 void
 QDjViewPrefs::load()
 {
+  // load recent files
+  loadRecent();
+  
+  // load everything else
   QSettings s;
-
   int versionFix = 0;
   if (s.contains("versionFix"))
     versionFix = s.value("versionFix").toInt();
-
   loadGroup(s, "forEmbeddedPlugin", forEmbeddedPlugin);
   loadGroup(s, "forFullPagePlugin", forFullPagePlugin);
   loadGroup(s, "forStandalone", forStandalone);
   loadGroup(s, "forFullScreen", forFullScreen);
-  
+  loadGroup(s, "forSlideShow", forSlideShow);
   if (s.contains("windowSize"))
     windowSize = s.value("windowSize").toSize();
+  if (s.contains("windowMaximized"))
+    windowMaximized = s.value("windowMaximized").toBool();
   if (s.contains("tools"))
     tools = stringToTools(s.value("tools").toString());
   if (versionFix < 0x40200)
@@ -339,10 +370,24 @@ QDjViewPrefs::load()
     browserProgram = s.value("browserProgram").toString();
   if (s.contains("proxyUrl"))
     proxyUrl = s.value("proxyUrl").toString();
+  if (s.contains("enableAnimations"))
+    enableAnimations = s.value("enableAnimations").toBool();
   if (s.contains("advancedFeatures"))
     advancedFeatures = s.value("advancedFeatures").toBool();
   if (s.contains("showTextLabel"))
     showTextLabel = s.value("showTextLabel").toBool();
+  if (s.contains("invertLuminance"))
+    invertLuminance = s.value("invertLuminance").toBool();
+  if (s.contains("mouseWheelZoom"))
+    mouseWheelZoom = s.value("mouseWheelZoom").toBool();
+  if (s.contains("language"))
+    languageOverride = s.value("language").toString();
+  if (s.contains("restrictOverride"))
+    restrictOverride = s.value("restrictOverride").toBool();
+  if (s.contains("openGLAccel"))
+    openGLAccel = s.value("openGLAccel").toBool();
+  if (s.contains("slideShowDelay"))
+    slideShowDelay = s.value("slideShowDelay").toInt();
   if (s.contains("modifiersForLens"))
     modifiersForLens 
       = stringToModifiers(s.value("modifiersForLens").toString());
@@ -352,19 +397,16 @@ QDjViewPrefs::load()
   if (s.contains("modifiersForLinks"))
     modifiersForLinks 
       = stringToModifiers(s.value("modifiersForLinks").toString());
-
   if (s.contains("thumbnailSize"))
     thumbnailSize = s.value("thumbnailSize").toInt();
   if (s.contains("thumbnailSmart"))
     thumbnailSmart = s.value("thumbnailSmart").toBool();
-
   if (s.contains("searchWordsOnly"))
     searchWordsOnly = s.value("searchWordsOnly").toBool();
   if (s.contains("searchCaseSensitive"))
     searchCaseSensitive = s.value("searchCaseSensitive").toBool();
   if (s.contains("searchRegExpMode"))
     searchRegExpMode = s.value("searchRegExpMode").toBool();
-  
   if (s.contains("infoDialogTab"))
     infoDialogTab = s.value("infoDialogTab").toInt();
   if (s.contains("metaDialogTab"))
@@ -380,8 +422,8 @@ QDjViewPrefs::load()
   if (s.contains("printReverse"))
     printCollate = s.value("printReverse").toBool();
 
-  if (s.contains("recentFiles"))
-    recentFiles = s.value("recentFiles").toStringList();
+  // advertise change
+  emit updated();
 }
 
 
@@ -397,27 +439,77 @@ QDjViewPrefs::saveGroup(QSettings &s, QString name, Saved &saved)
 }
 
 
-/*! Save all member variables for later use. */
+/*! Save recent files. */
 
 void
-QDjViewPrefs::save(void)
+QDjViewPrefs::saveRecent(void)
 {
   QSettings s;
-  
-  s.setValue("version", versionString());
+  s.setValue("recentFiles", recentFiles);
+}
 
+
+/*! Save remembered state. */
+
+void
+QDjViewPrefs::saveRemembered(void)
+{
+  QSettings s;
+
+  s.setValue("version", versionString());
   int versionFix = 0;
   if (s.contains("versionFix"))
     versionFix = s.value("versionFix").toInt();
   if (DJVIEW_VERSION > versionFix)
     s.setValue("versionFix", DJVIEW_VERSION);
-  
-  saveGroup(s, "forEmbeddedPlugin", forEmbeddedPlugin);
-  saveGroup(s, "forFullPagePlugin", forFullPagePlugin);
-  saveGroup(s, "forStandalone", forStandalone);
-  saveGroup(s, "forFullScreen", forFullScreen);
+
+  if (forEmbeddedPlugin.remember)
+    saveGroup(s, "forEmbeddedPlugin", forEmbeddedPlugin);
+  if (forFullPagePlugin.remember)
+    saveGroup(s, "forFullPagePlugin", forFullPagePlugin);
+  if (forStandalone.remember)
+    saveGroup(s, "forStandalone", forStandalone);
+  if (forFullScreen.remember)
+    saveGroup(s, "forFullScreen", forFullScreen);
+  if (forSlideShow.remember)
+    saveGroup(s, "forSlideShow", forSlideShow);
 
   s.setValue("windowSize", windowSize);
+  s.setValue("windowMaximized", windowMaximized);
+  s.setValue("thumbnailSize", thumbnailSize);
+  s.setValue("thumbnailSmart", thumbnailSmart);
+  s.setValue("searchWordsOnly", searchWordsOnly);
+  s.setValue("searchCaseSensitive", searchCaseSensitive);
+  s.setValue("searchRegExpMode", searchRegExpMode);
+  s.setValue("infoDialogTab", infoDialogTab);
+  s.setValue("metaDialogTab", metaDialogTab);
+  s.setValue("printerName", printerName);
+  s.setValue("printFile", printFile);
+  s.setValue("printCollate", printCollate);
+  s.setValue("printReverse", printReverse);
+}
+
+/*! Save all member variables for later use. */
+
+void
+QDjViewPrefs::save(void)
+{
+  // save parts
+  saveRecent();
+  saveRemembered();
+
+  // save everything else
+  QSettings s;
+  if (! forEmbeddedPlugin.remember)
+    saveGroup(s, "forEmbeddedPlugin", forEmbeddedPlugin);
+  if (! forFullPagePlugin.remember)
+    saveGroup(s, "forFullPagePlugin", forFullPagePlugin);
+  if (! forStandalone.remember)
+    saveGroup(s, "forStandalone", forStandalone);
+  if (! forFullScreen.remember)
+    saveGroup(s, "forFullScreen", forFullScreen);
+  if (! forSlideShow.remember)
+    saveGroup(s, "forSlideShow", forSlideShow);
   s.setValue("tools", toolsToString(tools));
   s.setValue("gamma", gamma);
   s.setValue("white", white.name());
@@ -428,41 +520,23 @@ QDjViewPrefs::save(void)
   s.setValue("lensPower", lensPower);
   s.setValue("browserProgram", browserProgram);
   s.setValue("proxyUrl", proxyUrl.toString());
+  s.setValue("enableAnimations", enableAnimations);
   s.setValue("advancedFeatures", advancedFeatures);
   s.setValue("showTextLabel", showTextLabel);
+  s.setValue("invertLuminance", invertLuminance);
+  s.setValue("mouseWheelZoom", mouseWheelZoom);
   s.setValue("modifiersForLens", modifiersToString(modifiersForLens));
   s.setValue("modifiersForSelect", modifiersToString(modifiersForSelect));
   s.setValue("modifiersForLinks", modifiersToString(modifiersForLinks));
-
-  s.setValue("thumbnailSize", thumbnailSize);
-  s.setValue("thumbnailSmart", thumbnailSmart);
-
-  s.setValue("searchWordsOnly", searchWordsOnly);
-  s.setValue("searchCaseSensitive", searchCaseSensitive);
-  s.setValue("searchRegExpMode", searchRegExpMode);
-  
-  s.setValue("infoDialogTab", infoDialogTab);
-  s.setValue("metaDialogTab", metaDialogTab);
+  s.setValue("language", languageOverride);
+  s.setValue("restrictOverride", restrictOverride);
+  s.setValue("openGLAccel", openGLAccel);
+  s.setValue("slideShowDelay", slideShowDelay);
   s.setValue("printerGamma", printerGamma);
-  s.setValue("printerName", printerName);
-  s.setValue("printFile", printFile);
-  s.setValue("printCollate", printCollate);
-  s.setValue("printReverse", printReverse);
-
-  s.setValue("recentFiles", recentFiles);
 }
 
 
 
-/*! Emit the updated() signal. */
-
-void
-QDjViewPrefs::update()
-{
-  emit updated();
-}
-
-
 /*! \fn QDjViewPrefs::updated() 
   This signal indicates that viewers should reload the preferences. */
 
@@ -615,7 +689,7 @@ QDjViewModifiersComboBox::QDjViewModifiersComboBox(QWidget *parent)
   keynames << "Shift";
   keynames << "Control";
   keynames << "Alt";
-#ifdef Q_WS_MAC
+#ifdef Q_OS_DARWIN
   keynames << "Command";
 #else
   keynames << "Meta";
@@ -714,7 +788,8 @@ struct QDjViewPrefsDialog::Private
 {
   QDjVuContext          *context;
   Ui::QDjViewPrefsDialog ui;
-  QDjViewPrefs::Saved    saved[4];
+  QDjViewPrefs::Saved    saved[5];
+  int                    slideShowDelay;
   int                    currentSaved;
 };
 
@@ -809,22 +884,13 @@ QDjViewPrefsDialog::QDjViewPrefsDialog()
              "</html>") );
   
   setHelp(d->ui.advancedTab,
-          tr("<html><b>Caches.</b>"
-             "<br>The <i>pixel cache</i> stores image data"
-             " located outside the visible area."
-             " This cache makes panning smoother."
-             " The <i>decoded page cache</i> contains partially"
-             " decoded pages. It provides faster response times"
-             " when navigating a multipage document or when returning"
-             " to a previously viewed page. Clearing this cache"
-             " might be useful to reflect a change in the page"
-             " data without restarting the DjVu viewer."
-
-             "<p><b>Miscellaneous.</b>"
-             "<br>Forcing a manual color correction can be useful"
-             " when using ancient printers."
-             " The advanced features check box enables a small number"
-             " of additional menu entries useful for authoring DjVu files."
+          tr("<html><b>Advanced.</b>"
+             "<br>You can override the default interface language, "
+             "disable the page animations, or enable additional "
+             "menu entries that are useful for authoring DjVu files."
+             "You can also disable the printing or saving restrictions "
+             "dictated by certain web sites. The manual color "
+             "correction can be useful with old printers."
              "</html>"));
 
   setHelp(d->ui.networkTab,
@@ -834,6 +900,16 @@ QDjViewPrefsDialog::QDjViewPrefsDialog()
              " a djvu document through a http url."
              " The djview plugin always uses the proxy"
              " settings of the web browser."
+             "<p><b>Cache settings.</b>"
+             "<br>The <i>pixel cache</i> stores image data"
+             " located outside the visible area."
+             " This cache makes panning smoother."
+             " The <i>decoded page cache</i> contains partially"
+             " decoded pages. It provides faster response times"
+             " when navigating a multipage document or when returning"
+             " to a previously viewed page. Clearing this cache"
+             " might be useful to reflect a change in the page"
+             " data without restarting the DjVu viewer."""
              "</html>") );
 }
 
@@ -843,11 +919,13 @@ QDjViewPrefsDialog::connectModified(QWidget *w)
 {
   if (w->inherits("QComboBox"))
     connect(w, SIGNAL(editTextChanged(QString)), this, SLOT(setModified()));
-  else if (w->inherits("QSpinBox"))
+  if (w->inherits("QComboBox"))
+    connect(w, SIGNAL(currentIndexChanged(int)), this, SLOT(setModified()));
+  if (w->inherits("QSpinBox"))
     connect(w, SIGNAL(valueChanged(int)), this, SLOT(setModified()));
-  else if (w->inherits("QAbstractButton"))
+  if (w->inherits("QAbstractButton"))
     connect(w, SIGNAL(toggled(bool)), this, SLOT(setModified()));
-  else if (w->inherits("QAbstractSlider"))
+  if (w->inherits("QAbstractSlider"))
     connect(w, SIGNAL(valueChanged(int)), this, SLOT(setModified()));
   QObject *child;
   foreach(child, w->children())
@@ -866,25 +944,36 @@ QDjViewPrefsDialog::load(QDjView *djview)
   int res = prefs->resolution;
   d->ui.forceResolutionCheckBox->setChecked(res > 0);
   d->ui.resolutionSpinBox->setValue((res>0) ? res : logicalDpiY());
+  d->ui.invertLuminanceCheckBox->setChecked(prefs->invertLuminance);
   // 2- interface tab
+  QComboBox *cb = d->ui.modeComboBox;
+  cb->clear();
   d->saved[0] = prefs->forStandalone;
+  cb->addItem(tr("Standalone Viewer"), 
+              QDjView::STANDALONE);
   d->saved[1] = prefs->forFullScreen;
-  d->saved[2] = prefs->forFullPagePlugin;
-  d->saved[3] = prefs->forEmbeddedPlugin;
+  cb->addItem(tr("Standalone Viewer (Full Screen mode)"), 
+              QDjView::STANDALONE_FULLSCREEN);
+  d->saved[2] = prefs->forSlideShow;
+  cb->addItem(tr("Standalone Viewer (Slideshow mode)"), 
+              QDjView::STANDALONE_SLIDESHOW);
+  d->saved[3] = prefs->forFullPagePlugin;
+  cb->addItem(tr("Full Page Plugin"),
+              QDjView::FULLPAGE_PLUGIN);
+  d->saved[4] = prefs->forEmbeddedPlugin;
+  cb->addItem(tr("Embedded Plugin"),
+              QDjView::EMBEDDED_PLUGIN);
+  d->slideShowDelay = prefs->slideShowDelay;
   djview->fillZoomCombo(d->ui.zoomCombo);
-  int n = 0;
-  if (djview->getViewerMode() == QDjView::EMBEDDED_PLUGIN)
-    n = 3;
-  else if (djview->getViewerMode() == QDjView::FULLPAGE_PLUGIN)
-    n = 2;
-  else if (djview->getFullScreen()) 
-    n = 1;
+  int n = qMax(0, cb->findData(djview->getViewerMode()));
   modeComboChanged(n);
-  d->ui.modeComboBox->setCurrentIndex(n);
+  cb->setCurrentIndex(n);
   // 3- keys tab
   d->ui.keysForLensCombo->setValue(prefs->modifiersForLens);
   d->ui.keysForSelectCombo->setValue(prefs->modifiersForSelect);
   d->ui.keysForLinksCombo->setValue(prefs->modifiersForLinks);
+  d->ui.wheelScrollButton->setChecked(! prefs->mouseWheelZoom);
+  d->ui.wheelZoomButton->setChecked(prefs->mouseWheelZoom);
   // 4- lens tab
   bool lens = (prefs->lensPower > 0) && (prefs->lensSize > 0);
   d->ui.lensEnableCheckBox->setChecked(lens);
@@ -905,22 +994,70 @@ QDjViewPrefsDialog::load(QDjView *djview)
       d->ui.proxyPasswordLineEdit->setText(url.password());
     }
   d->ui.proxyCheckBox->setChecked(proxy);
-  // 6- advanced tab
   qreal k256 = 256 * 1024;
   qreal k1024 = 1024 * 1024;
   d->ui.pixelCacheSpinBox->setValue(qRound(prefs->pixelCacheSize / k256));
   d->ui.pageCacheSpinBox->setValue(qRound(prefs->cacheSize / k1024));
+  // 6- advanced tab
   double pgamma = prefs->printerGamma;
+  loadLanguageComboBox(prefs->languageOverride);
+  d->ui.advancedCheckBox->setChecked(prefs->advancedFeatures);
+  d->ui.animationCheckBox->setChecked(prefs->enableAnimations);
+  d->ui.textLabelCheckBox->setChecked(prefs->showTextLabel);
+  d->ui.restrictOverrideCheckBox->setChecked(prefs->restrictOverride);
+  d->ui.openGLCheckBox->setChecked(prefs->openGLAccel);
   d->ui.printerManualCheckBox->setChecked(pgamma > 0);
   d->ui.printerGammaSpinBox->setValue((pgamma > 0) ? pgamma : 2.2);
-  d->ui.advancedCheckBox->setChecked(prefs->advancedFeatures);
-  d->ui.textLabelCheckBox->setChecked(prefs->showTextLabel);
   // no longer modified
   setWindowModified(false);
   d->ui.applyButton->setEnabled(false);
 }
 
 
+void
+QDjViewPrefsDialog::loadLanguageComboBox(QString lang)
+{
+  // supported languages
+  static const char *languages[] = { 
+    "cs","de","en","es","fr","it","ja","ru","uk",
+    "zh_CN","zh_TW",0 } ;
+  // get application
+  QComboBox *cb = d->ui.languageComboBox;
+  QCheckBox *db = d->ui.languageCheckBox;
+  QDjViewApplication *app = qobject_cast<QDjViewApplication*>(qApp);
+  db->setEnabled(app != 0);
+  cb->clear();
+  // prepare combobox
+  static struct { const char *src; const char *com; } thisLang[] = {
+    QT_TRANSLATE_NOOP3("Generic", "thisLanguage", "Name of THIS language") };
+  QString myLanguage = QApplication::translate("Generic", thisLang[0].src);
+  if (myLanguage == thisLang[0].src)
+    myLanguage = "English";
+  int index = -1;
+  if (app) 
+    for (int i=0; languages[i]; i++)
+      {
+        QString nlang = languages[i];
+        QString xlang = QString::null;
+        QTranslator *dTrans = new QTranslator();
+        QTranslator *qTrans = new QTranslator();
+        if (app->loadTranslators(QStringList(nlang), dTrans, qTrans))
+          xlang = dTrans->translate("Generic", thisLang[0].src);
+        else if (nlang == "en" || nlang.startsWith("en"))
+          xlang = "English";
+        if (lang == nlang && xlang.size() > 0)
+          index = cb->count();
+        if (index < 0 && xlang == myLanguage)
+          index = cb->count();
+        if (xlang.size() > 0)
+          cb->addItem(xlang, nlang);
+        delete dTrans;
+        delete qTrans;
+      }
+  db->setChecked(index >= 0 && lang.size() > 0);
+  cb->setCurrentIndex(index);
+}
+
 void
 QDjViewPrefsDialog::apply()
 {
@@ -931,17 +1068,30 @@ QDjViewPrefsDialog::apply()
   prefs->resolution = 0;
   if (d->ui.forceResolutionCheckBox->isChecked())
     prefs->resolution = d->ui.resolutionSpinBox->value();
+  prefs->invertLuminance = d->ui.invertLuminanceCheckBox->isChecked();
   // 2- interface tab
-  int n = d->ui.modeComboBox->currentIndex();
+  QComboBox *cb = d->ui.modeComboBox;
+  int n = cb->currentIndex();
   modeComboChanged(n);
-  prefs->forStandalone = d->saved[0];
-  prefs->forFullScreen = d->saved[1];
-  prefs->forFullPagePlugin = d->saved[2];
-  prefs->forEmbeddedPlugin = d->saved[3];
+  for (int i=0; i<5; i++) 
+    switch (cb->itemData(i).toInt()) 
+      {
+      case QDjView::STANDALONE:
+        prefs->forStandalone = d->saved[i]; break;
+      case QDjView::STANDALONE_SLIDESHOW:
+        prefs->forSlideShow = d->saved[i]; break;
+      case QDjView::STANDALONE_FULLSCREEN:
+        prefs->forFullScreen = d->saved[i]; break;
+      case QDjView::EMBEDDED_PLUGIN:
+        prefs->forEmbeddedPlugin = d->saved[i]; break;
+      case QDjView::FULLPAGE_PLUGIN:
+        prefs->forFullPagePlugin = d->saved[i]; break;
+      }
   // 3- keys
   prefs->modifiersForLens = d->ui.keysForLensCombo->value();
   prefs->modifiersForSelect = d->ui.keysForSelectCombo->value();
   prefs->modifiersForLinks = d->ui.keysForLinksCombo->value();
+  prefs->mouseWheelZoom = d->ui.wheelZoomButton->isChecked();
   // 4- lens tab
   bool lens = d->ui.lensEnableCheckBox->isChecked();
   prefs->lensPower = (lens) ? d->ui.lensPowerSpinBox->value() : 0;
@@ -956,19 +1106,28 @@ QDjViewPrefsDialog::apply()
       prefs->proxyUrl.setUserName(d->ui.proxyUserLineEdit->text());
       prefs->proxyUrl.setPassword(d->ui.proxyPasswordLineEdit->text());
     }
-  // 6- advanced tab
   prefs->pixelCacheSize = d->ui.pixelCacheSpinBox->value() * 256 * 1024;
   prefs->cacheSize = d->ui.pageCacheSpinBox->value() * 1024 * 1024;
+  // 6- advanced tab
+  prefs->languageOverride = QString::null;
+  if (d->ui.languageCheckBox->isChecked()) 
+    {
+      int n = d->ui.languageComboBox->currentIndex();
+      prefs->languageOverride = d->ui.languageComboBox->itemData(n).toString();
+    }
+  prefs->enableAnimations = d->ui.animationCheckBox->isChecked();
+  prefs->advancedFeatures = d->ui.advancedCheckBox->isChecked();
+  prefs->showTextLabel = d->ui.textLabelCheckBox->isChecked();
+  prefs->restrictOverride = d->ui.restrictOverrideCheckBox->isChecked();
+  prefs->openGLAccel = d->ui.openGLCheckBox->isChecked();
   prefs->printerGamma = 0;
   if (d->ui.printerManualCheckBox->isChecked())
     prefs->printerGamma = d->ui.printerGammaSpinBox->value();
-  prefs->advancedFeatures = d->ui.advancedCheckBox->isChecked();
-  prefs->showTextLabel = d->ui.textLabelCheckBox->isChecked();
   // broadcast change
   setWindowModified(false);
   d->ui.applyButton->setEnabled(false);
   prefs->save();
-  prefs->update();
+  emit prefs->updated();
 }
 
 
@@ -986,14 +1145,30 @@ QDjViewPrefsDialog::reset()
                                   QDjViewPrefs::SHOW_SIDEBAR);
   QDjViewPrefs::Options optS = QDjViewPrefs::SHOW_SCROLLBARS;
   QDjViewPrefs::Options optT = QDjViewPrefs::SHOW_TOOLBAR;
-  d->saved[0] = defsaved;
-  d->saved[1] = defsaved;
-  d->saved[1].options &= ~(optMSS|optS|optT);
-  d->saved[2] = defsaved;
-  d->saved[2].options &= ~(optMSS);
-  d->saved[3] = defsaved;
-  d->saved[3].options &= ~(optMSS|optT);
-  d->saved[3].remember = false;
+  QComboBox *cb = d->ui.modeComboBox;
+  for (int i=0; i<5; i++) 
+    {
+      d->saved[i] = defsaved;
+      switch (cb->itemData(i).toInt()) 
+        {
+        case QDjView::STANDALONE_SLIDESHOW:
+          d->saved[i].remember = false;
+          d->saved[i].zoom = QDjVuWidget::ZOOM_FITPAGE;
+        case QDjView::STANDALONE_FULLSCREEN:
+          d->saved[i].options &= ~(optMSS|optS|optT);
+        case QDjView::STANDALONE:
+          break;
+        case QDjView::EMBEDDED_PLUGIN:
+          d->saved[i].remember = false;
+          d->saved[i].options &= ~(optMSS|optT);
+          d->saved[i].zoom = 100;
+          break;
+        case QDjView::FULLPAGE_PLUGIN:
+          d->saved[i].options &= ~(optMSS);
+          break;
+        }
+    }    
+  d->slideShowDelay = 10;
   d->currentSaved = -1;
   modeComboChanged(d->ui.modeComboBox->currentIndex());
   // 3- keys tab
@@ -1006,13 +1181,16 @@ QDjViewPrefsDialog::reset()
   d->ui.lensSizeSpinBox->setValue(300);
   // 5- network tab
   d->ui.proxyCheckBox->setChecked(false);
-  // 6- advanced tab
   d->ui.pixelCacheSpinBox->setValue(1);
   d->ui.pageCacheSpinBox->setValue(10);
-  d->ui.printerManualCheckBox->setChecked(false);
-  d->ui.printerGammaSpinBox->setValue(2.2);
+  // 6- advanced tab
+  d->ui.languageCheckBox->setChecked(false);
+  d->ui.animationCheckBox->setChecked(true);
   d->ui.advancedCheckBox->setChecked(false);
   d->ui.textLabelCheckBox->setChecked(false);
+  d->ui.openGLCheckBox->setChecked(false);
+  d->ui.printerManualCheckBox->setChecked(false);
+  d->ui.printerGammaSpinBox->setValue(2.2);
 }
 
 
@@ -1054,7 +1232,8 @@ set_reset(QFlags<T> &x, T y, bool plus, bool minus=true)
 void 
 QDjViewPrefsDialog::modeComboChanged(int n)
 {
-  if (d->currentSaved >=0 && d->currentSaved <4)
+  QComboBox *cb = d->ui.modeComboBox;
+  if (d->currentSaved >=0 && d->currentSaved <5)
     {
       // save ui values
       QDjViewPrefs::Saved &saved = d->saved[d->currentSaved];
@@ -1090,9 +1269,12 @@ QDjViewPrefsDialog::modeComboChanged(int n)
         saved.zoom = zoom;
       else if (zoomIndex >= 0)
         saved.zoom = d->ui.zoomCombo->itemData(zoomIndex).toInt();
+      // delay
+      if (cb->itemData(d->currentSaved) == QDjView::STANDALONE_SLIDESHOW)
+        d->slideShowDelay = d->ui.slideShowSpinBox->value();
     }
   d->currentSaved = n;
-  if (d->currentSaved >=0 && d->currentSaved <4)
+  if (d->currentSaved >=0 && d->currentSaved <5)
     {
       // save modified flag
       bool modified = isWindowModified();
@@ -1121,6 +1303,10 @@ QDjViewPrefsDialog::modeComboChanged(int n)
         d->ui.zoomCombo->setEditText(QString("%1%").arg(saved.zoom));
       else if (zoomIndex >= 0)
         d->ui.zoomCombo->setEditText(d->ui.zoomCombo->itemText(zoomIndex));
+      // delay
+      d->ui.slideShowSpinBox->setValue(d->slideShowDelay);
+      d->ui.slideShowSpinBox->setVisible(d->currentSaved == 2);
+      d->ui.slideShowLabel->setVisible(d->currentSaved == 2);
       // reset modified flag
       setWindowModified(modified);
     }
