@@ -19,6 +19,7 @@
 # include "config.h"
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <libdjvu/miniexp.h>
@@ -40,7 +41,9 @@
 #include <QTimer>
 #include <QUrl>
 #include <QVector>
-
+#if QT_VERSION >= 0x50000
+# include <QUrlQuery>
+#endif
 #if DDJVUAPI_VERSION < 17
 # error "DDJVUAPI_VERSION>=17 is required !"
 #endif
@@ -105,7 +108,7 @@ QDjVuContext::callback(ddjvu_context_t *, void *closure)
     {
       qcontext->flag = true;
       QEvent *qevent = new QEvent(QEvent::User);
-	  QCoreApplication::postEvent(qcontext, qevent);
+      QCoreApplication::postEvent(qcontext, qevent);
     }
 }
 
@@ -493,7 +496,12 @@ QDjVuDocument::setUrl(QDjVuContext *ctx, QUrl url)
     cache = false;
   bool djvuopts = false;
   QPair<QString,QString> pair;
-  foreach(pair, url.queryItems())
+#if QT_VERSION >= 0x50000
+  QList<QPair<QString,QString> > qitems = QUrlQuery(url).queryItems();
+#else
+  QList<QPair<QString,QString> > qitems = url.queryItems();
+#endif
+  foreach(pair, qitems)
     {
       if (pair.first.toLower() == "djvuopts")
         djvuopts = true;
@@ -796,7 +804,7 @@ QDjVuPage::handle(ddjvu_message_t *msg)
       emit pageinfo();
       return true;
     case DDJVU_CHUNK:
-      emit chunk(QString::fromAscii(msg->m_chunk.chunkid));
+      emit chunk(QString::fromLatin1(msg->m_chunk.chunkid));
       return true;
     case DDJVU_RELAYOUT:
       emit relayout();
